/* main loop for slist test */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <strings.h>
#include <string.h>
#include "myvga.h"
#include "spec.h"
#include "slist.h"
#include "skel.h"
#include "solid.h"
#include "anaglyph.h"

/* read in slist, print slist */

void usage();
void init_vga(int xres, int yres);
void ana_init_vga(int xres, int yres);


extern void lexer(char *, camera *, simple_triangle_list *, simple_point_list *);
main ( int argc, char **argv)
{
	char *infile, do_skel, do_solid, do_anaglyph;
	camera world_camera;
	int xres, yres, arg_count;
        simple_triangle_list tlist, *temp_tri;
	simple_point_list plist;
	sl_screen *screen;
	FILE *logfile;
        real az, bz, cz, avgz, minz, maxz, sizz, scalez;
	
	logfile = fopen("log", "w");
	
	do_skel = do_solid = do_anaglyph = 'n';
	if (argc < 2) 
        	usage();
  	else
    	{
        	arg_count = 1;
        	while(arg_count < (argc))
        	{
                	if (strcmp(argv[arg_count],"-skel") == 0)
                	{
                        	arg_count++;
				do_skel = 'y';
                	} else if (strcmp(argv[arg_count],"-solid") == 0) {
                        	arg_count++;
				do_solid = 'y';
                	} else if (strcmp(argv[arg_count],"-anaglyph") == 0) {
                        	arg_count++;
				do_anaglyph = 'y';
			} else {
                       		infile = strdup(argv[arg_count++]);
                		}
        	} //while
	} // if usage

        lexer(infile, &world_camera, &tlist, &plist);

	xres = world_camera.xres;
	yres = world_camera.yres;
	world_camera.buffer = (unsigned char *) malloc( sizeof(unsigned char) * xres * yres);
	world_camera.make_m3();
	printf("main diag: \n");
	world_camera.print();
	printf("Triangle list:\n");
        //tlist.print();
        temp_tri = &tlist;
        maxz = -100.0 ; minz = 100.0;
        while (temp_tri != NULL) {
             az = temp_tri->tri->a->pt.y;
             bz = temp_tri->tri->b->pt.y;
             cz = temp_tri->tri->c->pt.y;
             avgz = (az + bz + cz) / 3.0;
             if (avgz > maxz) maxz = avgz;
             if (avgz < minz) minz = avgz;
             temp_tri = temp_tri->next;
        } // find max and min avg points to scale cmap
        if (maxz > minz) {
             temp_tri = &tlist;
             sizz = 1.0 / (maxz - minz);
             while (temp_tri != NULL) {
                     az = temp_tri->tri->a->pt.y;
                     bz = temp_tri->tri->b->pt.y;
                     cz = temp_tri->tri->c->pt.y;
                     avgz = (az + bz + cz) / 3.0;
                     scalez = (avgz - minz) * sizz;
                     temp_tri->tri->color = (int) (scalez * 254.) + 1;
                     //temp_tri->tri->color = (int) (rand() % 256);
                     fprintf(logfile, "Avg: %3.2g Col: %d \n", avgz,
                        temp_tri->tri->color);
                     temp_tri = temp_tri->next;
             }
        }
        fprintf(logfile, "Minz: %3.2g Maxz: %3.2g \n", minz, maxz);
	printf("Point list:\n");
	//plist.print();
	printf("xres %d, yres %d\n", xres, yres);
 
	screen = new sl_screen(xres, yres);
	//screen->print();
	init_vga(xres, yres);
	if (do_skel == 'y') {
		my_vga_clear();
		draw_skel(&world_camera, &tlist, &plist);
	}
	if (do_solid == 'y') {
		my_vga_clear();
		draw_solid(screen, &world_camera, &tlist, &plist);
	}
	if (do_anaglyph == 'y') {
		ana_init_vga(xres, yres);
		my_vga_clear();
		draw_anaglyph(screen, &world_camera, &tlist, &plist);
	}
	my_vga_setmode_TEXT();
	fprintf(logfile, "Number of sl_nodes: %d\n", screen->sl_node_count);
	fprintf(logfile, "Number of used nod: %d\n", screen->sl_used_count);
	fprintf(logfile, "Difference        : %d\n", screen->sl_node_count 
						- screen->sl_used_count);
	fclose(logfile);
}

void usage()
{
	fprintf(stderr, "usage: slist [-skel] [-solid] [-anaglyph] filename \n\twhere:\n\t-skel:\tdraw as skeletal line framework\n");
	fprintf(stderr, "\t-solid:\tdraw as solid slist processed triangles\n");
	fprintf(stderr, "\t-anaglyph:\tdraw as red and blue anaglyphic solid slist processed triangles\n");
	fprintf(stderr, "\tfilename:\tRFS file format input file\n");
	exit(0);
}
	
void init_vga(int xres, int yres) {
	int t, xres_1, yres_1;


	xres_1 = xres - 1;
	yres_1 = yres - 1;
         
#ifndef DJDIR    
	my_vga_setmode(G320x200x256);
#else
        my_vga_setmode(1);
#endif
        my_vga_drawline(  0,  0,xres_1,  0, 22);
        my_vga_drawline(  0,  0,  0,yres_1, 22);
        my_vga_drawline(xres_1,  0,xres_1,yres_1, 22);
        my_vga_drawline(  0,yres_1,xres_1,yres_1, 22);
        my_vga_setpalette(0, 0, 0, 0);  /* black */
        my_vga_setpalette(1, 0, 0, 32 );  /* lt blue */
        my_vga_setpalette(2, 0, 0 , 63);  /* med blue */
        my_vga_setpalette(3, 0, 0 , 99);  /* brt blue */

   //     for (t = 2; t < 100; t++) 
   //             my_vga_setpalette(t, t, t, t);  /* blue green =-) */
   //     for (t = 100; t < 200; t++) 
   //             my_vga_setpalette(t, t, t, t);  /* blue green =-) */
        for (t = 3; t < 255; t++) 
                my_vga_setpalette(t, 0, 16 +  t / 8, 0 );  /* blue green =-) */
                //my_vga_setpalette(t, rand() % 256, rand() % 256, rand() % 256 );  /* blue green =-) */
        my_vga_setcolor(100);
}
void ana_init_vga(int xres, int yres) {
	int redmul, bluemul, redadd, blueadd, t, xres_1, yres_1;


	xres_1 = xres - 1;
	yres_1 = yres - 1;
         
#ifndef DJDIR    
	my_vga_setmode(G320x200x256);
#else
        my_vga_setmode(1);
#endif
	// the palette is kinda weird, as I can only specify six bits
	// of any red,green.blue triple.  So I have 16 shades of red,
	// 16 shades of blue, and 265 - 16 - 16 shades of magenta
	// The index looks like:
	//   [red] [blue]
	//   7654   3210 
	// e.g.:
	//   0000   0010 = 0x02 = 2, index 2 is shade two of pure blue.
	//   0011   0000 = 0x30 = 48, index 48 is shade three of pure red.
	//   0011   0010 = 0x32 = 50, index 50 is magenta with shade three
	//				red and shade two of blue. 
	//  the t >> 4 and t % 4 modify t to produce the above palette
	//
	//  each r, g, b value must be between 0 .. 63
	// to correct for darkness, I changed t >> 4 * 4 to t >> 4 * 2 + 32
        for (t = 0; t < 256; t++ ) { 
		redmul = t >> 4;
		bluemul = t % 16;
		redadd =  (redmul > 0 ) ? 16 : 0;
		blueadd =  (bluemul > 0 ) ? 16 : 0;
                my_vga_setpalette(t, redmul * 2 + redadd, 0 , 
			bluemul * 2 + blueadd );  /* red, blue, magenta */
	}
       // my_vga_setpalette(0,0,0,0);  /* black*/
        my_vga_setcolor(100);
}
