/* scanner for toy sdl */

%{
/* need this for atof() */
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "spec.h" /* for parsing */
#include "slist.h" /* for building */

void lexer(char *, camera *, simple_triangle_list *, simple_point_list *);
void parser(int, char *);
void p_rule_0(int arg);
void p_rule_1(int arg, char *val);
void p_rule_2(int arg, char *val);
void p_rule_3(int arg, char *val);
%}


DIGIT[0-9]
ID [a-zA-Z][A-Za-z0-9]*

%%

"-"*{DIGIT}+	{
	parser(2, yytext);
	}

"-"*{DIGIT}+"."{DIGIT}*	{
	parser(1, yytext);
	}
POINT	{
	//printf("A Point!\n ");
	parser(2, yytext);
	}
TRIANGLE {
	//printf("A Triangle\n");
	parser(3, yytext);
	}
CAM	{
	//printf("The Camera!\n");
	parser(1, yytext);
	}
{ID}	parser(3, yytext);

[ \t\n]+	/* eat up white space */

"//".*$		/* eat comments */

. printf(" what the hell is this??? ( %s ) ", yytext);

%%
typedef struct lex_point_tag {
	char *label;
	point *coord;
	struct lex_point_tag *next;
} lex_point;

lex_point *tmp_point, *point_list;
	
int p_rule, p_arg, num_points, num_triangles;
camera *tmp_cam;
simple_triangle_list *tmp_stl;
simple_point_list *tmp_spl;
itriangle *tmp_triangle;


/* p_rule 1 is the camer w/ 9 args
   p_rule 2 is the point w/ 4 args
   p_rule 3 is the triangle with 3 args

   if p_rule != 0, parser arg means
	1: val is float
	2: val is int
	3: val is char *
   
*/
void lex_add (lex_point *point)
{
	lex_point *save;
	simple_point_list *splsave;

	splsave = tmp_spl;
	tmp_spl = new simple_point_list;
	tmp_spl->p = point->coord;
	tmp_spl->next =splsave;

	save = point_list;
	point_list = point;
	point_list->next = save;
}


point *lex_match (char *name)
{
	point *result;
	lex_point *list;
	
	list = point_list;

	while (list != NULL) {
		if (strcmp(name, list->label) == 0) {
			result = list->coord;
			break;
		} else
			list = list->next;
	}
	if (list == NULL)
		printf("Unknown point label: (%s)\n", name);
	return result;
}
	

void p_rule_0(int arg)
{
	switch (arg) {	
		case 1: printf("parser detects camera\n");
			p_rule = 1; /* rule 1 is the camera */
			p_arg = 1;
			break;
		case 2: //printf("parser detects point\n");
			p_rule = 2; /* rule 2 is the point */
			p_arg = 1;
			num_points++;
			tmp_point = (lex_point *) (malloc (sizeof(lex_point))); 
			tmp_point->coord = new point;
			break;
		case 3: //printf("parser detects triangle\n");
			p_rule = 3; /* rule 3 is the triangle */
			p_arg = 1;
			num_triangles++;
                        tmp_triangle = new itriangle;
			break;
		default: printf("Syntax error!\n");
		} 					/* end of switch arg */
}


void parser (int arg, char *val)
{
	if ((arg == 0) && (val == NULL))
		p_rule = p_arg = 0;
	else
		switch (p_rule) {	
			case 0: p_rule_0(arg);	
				break;
			case 1:	p_rule_1(arg, val);
             			break;
			case 2:	p_rule_2(arg, val);
             			break;
			case 3:	p_rule_3(arg, val);
             			break;
           		default: printf("syntax error 8888 \n");
			} /* end of switch p_rule */
} /* parser */	

void p_rule_1(int arg, char *val) {
	switch (p_arg) {
		case 1: if (arg != 1)
			  	printf("syntax 1p error in cam def\n"); 
			else
		   	  	tmp_cam->position.x = atof(val);
			break;
		case 2: if (arg != 1)
				printf("syntax 2p error in cam def\n"); 
			else
			   	tmp_cam->position.y = atof(val);
			break;
		case 3: if (arg != 1)
				printf("syntax 3p error in cam def\n"); 
			else
		   		tmp_cam->position.z = atof(val);
			break;
		case 4: if (arg != 1)
			  	printf("syntax 1error in cam def\n"); 
			else
		   	  	tmp_cam->forward.x = atof(val);
			break;
		case 5: if (arg != 1)
				printf("syntax 2error in cam def\n"); 
			else
			   	tmp_cam->forward.y = atof(val);
			break;
		case 6: if (arg != 1)
				printf("syntax 3error in cam def\n"); 
			else
		   		tmp_cam->forward.z = atof(val);
			break;
		case 7: if (arg != 1)
				printf("syntax 4error in cam def\n"); 
			else
		   		tmp_cam->up.x = atof(val);
			break;
		case 8: if (arg != 1)
				printf("syntax 5error in cam def\n"); 
			else
		   		tmp_cam->up.y = atof(val);
			break;
		case 9: if (arg != 1)
				printf("syntax 6error in cam def\n"); 
			else
		   		tmp_cam->up.z = atof(val);
			break;
		case 10: if (arg != 2)
				printf("syntax 7error in cam def\n"); 
			else
		   		tmp_cam->xres = atoi(val);
			break;
		case 11: if (arg != 2)
				printf("syntax 8error in cam def\n"); 
			else
		   		tmp_cam->yres = atoi(val);
			break;
		case 12: if (arg != 1)
				printf("syntax 9error in cam def\n"); 
			else
		   		tmp_cam->zoom = atof(val);
			p_rule = 0;
			break;
		default: printf("syntax 10error in cam def\n"); 
		} 					/* end switch p_arg */
	p_arg++;
}

/* point:  point label x y z */
void p_rule_2(int arg, char *val) {
	switch (p_arg) {
		case 1: if (arg != 3)
			  	printf("syntax 1error in point def\n"); 
			else	{
		   	  	tmp_point->label = strdup(val);
				tmp_point->coord->label = strdup(val);
				}
			break;
		case 2: if (arg != 1)
				printf("syntax 2error in point def\n"); 
			else
			   	tmp_point->coord->pt.x = atof(val);
			break;
		case 3: if (arg != 1)
				printf("syntax 3error in point def\n"); 
			else
		   		tmp_point->coord->pt.y = atof(val);
			break;
		case 4: if (arg != 1)
				printf("syntax 4error in point def\n"); 
			else
		   		tmp_point->coord->pt.z = atof(val);
			lex_add ( tmp_point);
			p_rule = 0;
			break;
		default: printf("syntax 10error in point def\n"); 
		} 					/* end switch p_arg */
	p_arg++;
}

/* Triangle label label label */
void p_rule_3(int arg, char *val) {
	simple_triangle_list *p;
	switch (p_arg) {
		case 1: if (arg != 3)
			  	printf("syntax 1error in triangle def\n"); 
			else
		   	  	tmp_triangle->a = lex_match(val);
			break;
		case 2: if (arg != 3)
				printf("syntax 2error in triangle def\n"); 
			else
			   	tmp_triangle->b = lex_match(val);
			break;
		case 3: if (arg != 3)
				printf("syntax 3error in triangle def\n"); 
			else
		   		tmp_triangle->c = lex_match(val);
			tmp_triangle->normal = v3_cross ( 
					tmp_triangle->a->pt -
					tmp_triangle->c->pt ,
					tmp_triangle->b->pt -
					tmp_triangle->c->pt);
			
			tmp_triangle->color = num_triangles;
			p = tmp_stl;
			tmp_stl = new simple_triangle_list;
			tmp_stl->next = p;
			tmp_stl->tri = tmp_triangle;
			p_rule = 0;
			break;
		default: printf("syntax 10error in triangle def\n"); 
		} 					/* end switch p_arg */
	p_arg++;
}


int yywrap() {
	return 1;
}

void lexer (char *infile, camera *worldcam, simple_triangle_list *stl,
					simple_point_list *spl)
	{
	/* skip program name */
	num_points = num_triangles =0;
	if ( infile != NULL  )
		yyin = fopen( infile, "r");
	else
		yyin = stdin;
	parser(0, NULL);  /* init parser */
	tmp_cam = worldcam;
	point_list = tmp_point = NULL;
	tmp_triangle = NULL;
	tmp_stl = NULL;
	tmp_spl = NULL;
	yylex();
	*stl = *tmp_stl;
	*spl = *tmp_spl;
		printf("Number of points: %d \n", num_points);
		printf("Number of triangles: %d\n", num_triangles);
	}

