#ifndef _SLIST_H
#define _SLIST_H

#include "matrix.h"
#include "spec.h"

#define SL_BIG 10000.0
#define CONSIDERATION_LIST_SIZE 50
#define ENDPOINT_LIST_SIZE 5000

/* hello */

/* a screen is a collection of scanlines.
   Each scanline will be rendered using an
   slist.

	tri_list -(get_scanlines)-> scanline segments -(emit_segment)-> 
	unsorted list -(tree sort)-) endpoint list -(kahuna)-> sorted list 
	-(shading engine)-> video buffer
*/


typedef struct sl_node_tag {
	struct sl_node_tag *next;
	int startpixel, endpixel;
        itriangle *tri;
	v3 *in, *out;
	float zin, zout, zdelta;
	print();
} sl_node;

typedef struct endpoint_node_tag {
	bool head_or_tail;
	sl_node *segment;
	int xpixel;
} endpoint_node;

class sl_scanline  {
  public:
	sl_node *unsorted, *sorted, *unsorted_tail, *sorted_tail;
	sl_scanline *next;
	int index;
	print();
};

class sl_screen {	
  public:
	int xres, yres, sl_node_count, sl_used_count, 
		endpoint_list_length, consideration_list_length;
	sl_scanline **scanlines;
	sl_node *sl_heap;
	endpoint_node endpoints[ENDPOINT_LIST_SIZE];
	int endpoint_xvals[ENDPOINT_LIST_SIZE];
	int endpoint_idx[ENDPOINT_LIST_SIZE];
	endpoint_node *kahuna;
	sl_node  * consideration[CONSIDERATION_LIST_SIZE];

	sl_screen(int,int); // xres,yres as input
	void sl_screen_add_unsorted( int iscanline, int i_in_x, int i_out_x, 
                        float inz, float outz, itriangle *itri);
	void sl_screen_add_sorted( int iscanline, int i_in_x, int i_out_x, 
                        float inz, float outz, itriangle *itri);
	void free_unsorted_sl_nodes(sl_scanline *sc);
	void free_sorted_sl_nodes();
	void draw_unsorted();
	void draw_sorted();
	void draw_buffer(camera *);
	void draw_buffer_left_right(camera *, char left_right);
	print();
};



void get_scanlines(sl_screen *, itriangle *tri, int xres, int yres);
void emit_segment(sl_screen *, int, int, int, int, float, float, itriangle *);
	         // screen, scanline, inx, outx, xres, inz, outz, tri
//void kahuna(sl_screen *);
void consideration_list_add(int listsize, sl_node **con_list,  sl_node *seg);
void consideration_list_remove(int listsize, sl_node **con_list, sl_node *seg);
sl_node * consideration_maxify(int listsize, sl_node **con_list);
float segheight(sl_node *seg, int x);
void endpoint_sort(sl_screen *);

#endif

//  SLIST STRUCTURE:
//
//	 scanlines[0]         sorted/unsorted	    next              next
//  sl_screen ------> sl_scanline -------> sl_node -----> sl_node -...---> NULL
//	|		|
//	|		| next
//	|scanlines[1]	V      sorted/unsorted	    next              next
//      +-----------> sl_scanline -------> sl_node -----> sl_node -...---> NULL
//	|		|
//	|		| next
//	|scanlines[2]	V       sorted/unsorted	    next              next
//      +-----------> sl_scanline -------> sl_node -----> sl_node -...---> NULL
//      |		|
//      .		.
//	.		.
//	.		.
//      |		|
//	|scanlines[yres]V       sorted/unsorted	   next               next
//      +-----------> sl_scanline -------> sl_node -----> sl_node -...---> NULL
//

