/* $Id: spec.h,v 1.8 1998/10/03 05:18:59 sean Exp sean $ */

#ifndef SPEC_H
#define SPEC_H

#include "matrix.h"

class point {
public:
	char *label;
        v3 pt, *trans;
	d2 project;
	char drawable;

        point();
	print();
};
	
//
// The camera is best described as a forward vector and an up vector,
// which are assumed to be perpendicular, with a rectangular piece of
// film with resolution xres, yres.  zoom controls the camera zoom.
// m is the matrix for the camera space with the forward vector becoming z,
// the up vector becoming y, and the cross product of foreward and up
// becoming x (a.k.a. fxu).
// inverse is m^-1, and m_inv_pos is (m^-1)*position.  This is for
// the transformation of a point in world coordinates to camera coordinates.
//
// e.g. for point p, the transformed point is t, with the origin
// at the camera position, and the axes described above:
//    [p - position] = [m] * t
//
//    rearranged:
//			t = [m]^-1 * p - [m]^-1 * position
//				         ^^^^m_inv_pos^^^^


class camera {
public:
  v3 position, forward, up, fxu, m_inv_pos;
  m3 m, inverse;
  int  xres, yres;
  real zoom;
  unsigned char *buffer;  // film

  make_m3();
  void  cam_trans(v3 *p, v3 *result );  // transformation from world to camera coordinates
  void  cam_proj(v3 *p, d2 *result);  // camera projection to film
  print();
};

class itriangle {
public:
  point *a, *b, *c;
  v3  normal;  // later change to v3 normal(v2 intersection)
  int color;
  print();
};

class light {
public:
  v3  *direction;  // just directional lighting for now
  v3  *color;
  print();
};

class simple_triangle_list {
public:
        //triangle *tri;
        simple_triangle_list *next;
        itriangle *tri;
        print();
};

class simple_point_list {
  public:
	point *p;
	simple_point_list *next;
	print();
};
#endif
